@@ -1,11 +1,15 @@
+# -*- coding: utf-8 -*-
+from typing import Dict
+from typing import List
+
 from cleo import argument
 from cleo import option
 
-from .env_command import EnvCommand
 from .init import InitCommand
+from .installer_command import InstallerCommand
 
 
-class AddCommand(EnvCommand, InitCommand):
+class AddCommand(InstallerCommand, InitCommand):
 
     name = "add"
     description = "Adds a new dependency to <comment>pyproject.toml</>."
@@ -33,12 +37,19 @@ class AddCommand(EnvCommand, InitCommand):
             "Platforms for which the dependency must be installed.",
             flag=False,
         ),
+        option(
+            "source",
+            None,
+            "Name of the source to use to install the package.",
+            flag=False,
+        ),
         option("allow-prereleases", None, "Accept prereleases."),
         option(
             "dry-run",
             None,
             "Output the operations but do not execute anything (implicitly enables --verbose).",
         ),
+        option("lock", None, "Do not perform operations (only update the lockfile)."),
     ]
     help = (
         "The add command adds required packages to your <comment>pyproject.toml</> and installs them.\n\n"
@@ -53,13 +64,13 @@ class AddCommand(EnvCommand, InitCommand):
         "  - A url (<b>https://example.com/packages/my-package-0.1.0.tar.gz</b>)\n"
     )
 
-    loggers = ["poetry.repositories.pypi_repository"]
+    loggers = ["poetry.repositories.pypi_repository", "poetry.inspection.info"]
 
     def handle(self):
-        from poetry.installation.installer import Installer
-        from poetry.semver import parse_constraint
         from tomlkit import inline_table
 
+        from poetry.core.semver import parse_constraint
+
         packages = self.argument("name")
         is_dev = self.option("dev")
 
@@ -79,21 +90,24 @@ class AddCommand(EnvCommand, InitCommand):
         if section not in poetry_content:
             poetry_content[section] = {}
 
-        for name in packages:
-            for key in poetry_content[section]:
-                if key.lower() == name.lower():
-                    pair = self._parse_requirements([name])[0]
-                    if (
-                        "git" in pair
-                        or "url" in pair
-                        or pair.get("version") == "latest"
-                    ):
-                        continue
+        existing_packages = self.get_existing_packages_from_input(
+            packages, poetry_content, section
+        )
 
-                    raise ValueError("Package {} is already present".format(name))
+        if existing_packages:
+            self.notify_about_existing_packages(existing_packages)
+
+        packages = [name for name in packages if name not in existing_packages]
+
+        if not packages:
+            self.poetry.file.write(content)
+            self.line("Nothing to add.")
+            return 0
 
         requirements = self._determine_requirements(
-            packages, allow_prereleases=self.option("allow-prereleases")
+            packages,
+            allow_prereleases=self.option("allow-prereleases"),
+            source=self.option("source"),
         )
 
         for _constraint in requirements:
@@ -130,6 +144,9 @@ class AddCommand(EnvCommand, InitCommand):
             if self.option("platform"):
                 constraint["platform"] = self.option("platform")
 
+            if self.option("source"):
+                constraint["source"] = self.option("source")
+
             if len(constraint) == 1 and "version" in constraint:
                 constraint = constraint["version"]
 
@@ -144,16 +161,17 @@ class AddCommand(EnvCommand, InitCommand):
         # Update packages
         self.reset_poetry()
 
-        installer = Installer(
-            self.io, self.env, self.poetry.package, self.poetry.locker, self.poetry.pool
-        )
+        self._installer.set_package(self.poetry.package)
+        self._installer.dry_run(self.option("dry-run"))
+        self._installer.verbose(self._io.is_verbose())
+        self._installer.update(True)
+        if self.option("lock"):
+            self._installer.lock()
 
-        installer.dry_run(self.option("dry-run"))
-        installer.update(True)
-        installer.whitelist([r["name"] for r in requirements])
+        self._installer.whitelist([r["name"] for r in requirements])
 
         try:
-            status = installer.run()
+            status = self._installer.run()
         except Exception:
             self.poetry.file.write(original_content)
 
@@ -162,12 +180,37 @@ class AddCommand(EnvCommand, InitCommand):
         if status != 0 or self.option("dry-run"):
             # Revert changes
             if not self.option("dry-run"):
-                self.error(
+                self.line_error(
                     "\n"
-                    "Addition failed, reverting pyproject.toml "
-                    "to its original content."
+                    "<error>Failed to add packages, reverting the pyproject.toml file "
+                    "to its original content.</error>"
                 )
 
             self.poetry.file.write(original_content)
 
         return status
+
+    def get_existing_packages_from_input(
+        self, packages, poetry_content, target_section
+    ):  # type: (List[str], Dict, str) -> List[str]
+        existing_packages = []
+
+        for name in packages:
+            for key in poetry_content[target_section]:
+                if key.lower() == name.lower():
+                    existing_packages.append(name)
+
+        return existing_packages
+
+    def notify_about_existing_packages(
+        self, existing_packages
+    ):  # type: (List[str]) -> None
+        self.line(
+            "The following packages are already present in the pyproject.toml and will be skipped:\n"
+        )
+        for name in existing_packages:
+            self.line("  • <c1>{name}</c1>".format(name=name))
+        self.line(
+            "\nIf you want to update it to the latest compatible version, you can use `poetry update package`.\n"
+            "If you prefer to upgrade it to the latest available version, you can use `poetry add package@latest`.\n"
+        )
